import re

from .common import InfoExtractor
from ..utils import (
    int_or_none,
    parse_iso8601,
    traverse_obj,
    unified_strdate,
)


class RadioplayRedirectIE(InfoExtractor):
    _VALID_URL = r'https?://(?:live|lytte)\.radioplay\.(?:se|no|dk)/(?P<id>\d+)'

    _TESTS = [{
        'url': 'https://lytte.radioplay.no/10279674',
        'info_dict': {
            'id': '10030542',
            'display_id': '2035659',
            'ext': 'mp3',
            'title': 'Best of Høsten 2020',
            'description': 'md5:ea1af250826e1875931754fbdf51aa53',
            'timestamp': 1603429200,
            'upload_date': '20201023',
            'channel': 'Siri og de gode hjelperne',
            'thumbnail': r're:https?://.*',
            'duration': 4182,
            'season': 'Season 6',
            'season_number': 6,
        },
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        webpage = self._download_webpage(url, video_id)
        url = self._search_regex(
            r'desktopUrl\s*:\s*\'([^\']+)\'', webpage,
            'redirect', video_id)
        return self.url_result(url)


class RadioplayIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?radioplay\.(?:se|no|dk)/[^/]+/spiller/(?P<id>\d+)/?'

    _TESTS = [{
        'url': 'https://radioplay.no/radio-rock/spiller/178851646/',
        'info_dict': {
            'id': '178851646',
            'ext': 'mp3',
            'title': 'Stå Opp!',
            'timestamp': 1630141200,
            'upload_date': '20210828',
        },
    }]

    def _extract_player(self, url):
        display_id = self._match_id(url)
        webpage = self._download_webpage(url, display_id)
        nextdata = self._search_nextjs_data(webpage, display_id)
        video_info = nextdata['props']['initialState']['audibles']['data'][0]

        return (
            display_id,
            video_info,
        )

    def _real_extract(self, url):
        display_id, video_info = self._extract_player(url)
        return {
            'url': video_info['mediaUrl'] or video_info['mediaUrl_mp3'],
            'id': str(video_info['id']),
            'display_id': display_id,
            'title': video_info['title'],
            'description': video_info.get('description'),
            'season_number': video_info['seasonNumber'],
            'thumbnail': video_info.get('image'),
            'timestamp': parse_iso8601(video_info.get('published_at'), ' '),
            'duration': int_or_none(video_info.get('duration')),
            # fishy!:
            'channel': video_info.get('showTitle'),
        }


class RadioplayPodcastRedirectIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?radioplay\.(?:se|no|dk)/podcast/[^/]+/id-(?P<id>\d+)'

    def _real_extract(self, url):
        video_id = self._match_id(url)
        webpage = self._download_webpage(url, video_id)
        for url in re.findall(r'<div[^>]* data-test="audible-now-playing-button-container"[^>]*>[^<]*<a href="([^"]+)"', webpage):
            return self.url_result(url)


class RadioplayPodcastIE(RadioplayIE):
    _VALID_URL = r'https?://(?:www\.)?radioplay\.(?:se|no|dk)/podcast/[^/]+/[^/]+/(?P<id>\d+)'

    _TESTS = [{
        'url': 'https://radioplay.se/podcast/lilla-my/lyssna/2001126',
        'info_dict': {
            'id': '10027569',
            'display_id': '2001126',
            'ext': 'mp3',
            'title': 'Kaktus till läraren',
            'description': 'Lilla My ska köpa en "blomma" till sin lärare.',
            'timestamp': 1549898100,
            'upload_date': '20190211',
            'channel': 'Lilla My',
            'duration': 114,
            'thumbnail': 'https://planet-radio-studio-podplay.imgix.net/img/b79602e6910c0bb002f2317995352ca6.png?auto=compress&h=1200&w=1200',
        },
    }]

    def _real_extract(self, url):
        display_id, video_info = self._extract_player(url)
        return {
            'url': video_info['mediaUrl'] or video_info['mediaUrl_mp3'],
            'id': str(video_info['id']),
            'display_id': display_id,
            'title': video_info['title'],
            'description': video_info.get('description'),
            'season_number': video_info['seasonNumber'],
            'thumbnail': video_info.get('image'),
            'timestamp': parse_iso8601(video_info.get('published_at'), ' '),
            'duration': int_or_none(video_info.get('duration')),
            # fishy!:
            'channel': video_info.get('showTitle'),
        }
